/* This file is part of PolyCalc.

   PolyCalc is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   any later version.

   PolyCalc is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PolyCalc.  If not, see <http://www.gnu.org/licenses/>.

   For more information about PolyCalc see project home
   <https://launchpad.net/polycalc>.

   Copyright 2012 PolyCalc development team */

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <algorithm>
#include <QPushButton>
#include <QLabel>
#include <QLineEdit>
#include <QMessageBox>
#include <QDebug>
#include <QFileDialog>
#include <QFileInfo>
#include <QIcon>
#include <QtXml>
#include <QErrorMessage>
#include <QtGlobal>
#include <QDesktopServices>
#include <QUrl>
#include "pcconfig.h"
#include "pclineedit.h"
#include "pclabel.h"
#include "pcbutton.h"
#include "pcgrid.h"
#include "pcmenu.h"
#include "pcqscriptengine.h"
#include "pcexecscriptengine.h"
#include "pcstatusbar.h"
#include "pcupdate.h"

MainWindow::MainWindow(QString layoutName, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    menuSignalMapper   = new QSignalMapper(this);
    settings           = new QSettings("PolyCalc development team", "PolyCalc");
    layout = NULL;
    ui->setupUi(this);
    ui->dockWidget->hide();
    ui->menuHelp->addAction(new pcUpdate());
    QWidget::setWindowIcon(QIcon(pcConfig::iconDir + "/polycalc.png"));
    PopulateMenu();
    loadLayout(layoutName.isNull()
               ? settings->value("layout", "basic").toString()
               : layoutName);
    ((pcStatusBar *)statusBar())->setStatusText("started");
    ui->dockWidget->setVisible(settings->value("command_log_visible",false).toBool());
    ui->actionCommand_Log->setChecked(settings->value("command_log_visible",false).toBool());
    resize(settings->value("window_width", 0).toInt(),
           settings->value("window_height", 0).toInt());
}

void MainWindow::on_scriptEngineError(QString error)
{
    ui->plainTextEdit->appendPlainText(error);
}

void MainWindow::CreateLayoutMenuItems(QString path)
{
    QDir dir(path);
    QStringList list = dir.entryList(QStringList("*.cal"));
    QStringListIterator iterator(list);
    while (iterator.hasNext()) {
        QString name = iterator.next();
        QString file = path + "/" + name;
        name.chop(4);
        layoutFiles[name] = file;
        QAction *action = new QAction(name, this);
        ui->menuOptions->addAction(action);
        menuSignalMapper->setMapping(action, name);
        connect(action, SIGNAL(triggered()),
                menuSignalMapper, SLOT (map()));
    }
}

void MainWindow::PopulateMenu()
{
    CreateLayoutMenuItems(pcConfig::layoutDir);
    CreateLayoutMenuItems(pcConfig::usersLayoutDir);
    connect(menuSignalMapper, SIGNAL(mapped(const QString &)),
            this, SLOT(loadLayout(const QString &)));
}

void MainWindow::resizeEvent(QResizeEvent *event)
{
    settings->setValue("window_width", width());
    settings->setValue("window_height", height());
}

void MainWindow::loadLayout(const QString &name)
{
    settings->setValue("layout",name);
    this->setWindowTitle("PolyCalc - "+name);

    if (layout != NULL) {
        ui->verticalLayout->removeItem(layout->getRootGrid());
        delete layout;
    }
    layout = new pcLayout(layoutFiles[name],this);

    QList<pcMenu *>::iterator m;
    for (m = layout->getMenus().begin();
         m < layout->getMenus().end(); ++m) {
        ui->menuBar->addMenu(*m);
    }
    ui->verticalLayout->addLayout(layout->getRootGrid());
    this->setStatusBar(layout->getStatusbar());
    ui->menuTools->addMenu(layout->getMacro());
    ui->verticalLayout->insertWidget(0,layout->getLineEdit());
    layout->getLineEdit()->setFocus();

    connect(layout->getScriptEngine(), SIGNAL(errorOccured(QString)),
            this, SLOT(on_scriptEngineError(QString)));
    connect(layout, SIGNAL(output(QString)),
            ui->plainTextEdit, SLOT(appendPlainText(QString)));

    QCoreApplication::sendPostedEvents();
    resize(0,0);
}

void MainWindow::on_actionQuit_triggered()
{
    this->close();
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox msgBox;
    msgBox.setTextFormat(Qt::RichText);
    msgBox.about(this,
                 "About",
                 "<b>PolyCalc v" + pcConfig::version + "</b><br />" +
                 "Configurable UI Calculator<br />" +
                 QString::fromUtf8("© 2012 PolyCalc Development Team<br />") +
                 "Distributed under GNU GPL v3<br />" +
                 "<a href='http://"+pcConfig::prjUrl+"'>"
                 +pcConfig::prjUrl+"</a>");
}



void MainWindow::on_actionHelp_triggered()
{
     QDesktopServices::openUrl(QUrl(pcConfig::helpUrl, QUrl::TolerantMode));
}

void MainWindow::on_actionCommand_Log_triggered()
{
    ui->dockWidget->setVisible(ui->actionCommand_Log->isChecked());
    settings->setValue("command_log_visible",ui->actionCommand_Log->isChecked());
}

void MainWindow::on_actionClear_command_log_triggered()
{
    ui->plainTextEdit->clear();
}

void MainWindow::on_actionLayout_help_triggered()
{
    layout->showHelp();
}

MainWindow::~MainWindow()
{
    delete layout;
    delete ui;
}

void MainWindow::on_actionSuggest_new_feature_triggered()
{
    QDesktopServices::openUrl(QUrl(pcConfig::wishlistUrl));
}

void MainWindow::on_actionLook_for_more_triggered()
{
    QDesktopServices::openUrl(QUrl(pcConfig::galeryUrl, QUrl::TolerantMode));
}
