/* This file is part of PolyCalc.

   PolyCalc is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   any later version.

   PolyCalc is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PolyCalc.  If not, see <http://www.gnu.org/licenses/>.

   For more information about PolyCalc see project home
   <https://launchpad.net/polycalc>.

   Copyright 2012 PolyCalc development team */

#include <QMessageBox>
#include <QErrorMessage>
#include "pclayout.h"
#include "pcbutton.h"
#include "pclabel.h"
#include "pcqscriptengine.h"

pcLayout::pcLayout(QString filename, QWidget *parent) :
    QObject((QObject *)parent)
{
    QDomDocument *doc    = readDomFromFile(filename);
    QDomElement polycalc = doc->documentElement();
    enter = pcButton::Enter();
    enter->hide();
    enter->setParent(parent);
    layouthelp           = "No help provided.";
    stringPipeEngine     = new pcAppScript();
    lineEdit             = new pcLineEdit(stringPipeEngine);
    statusbar            = NULL;
    scriptEngine         = NULL;
    macro = new pcMacro(parent);
    connect(macro, SIGNAL(sendButtonClick(QObject*)),
            this,  SLOT(anyButtonClicked(QObject*)));

    for (QDomNode rootNode = polycalc.firstChild();
         !rootNode.isNull();
         rootNode = rootNode.nextSibling())
    {
        if (rootNode.toElement().tagName() == "help") {
            layouthelp = rootNode.toElement().text();
        }
        if (rootNode.toElement().tagName() == "menu")
        {   // todo: scriptengine may not initialised
            menus << new pcMenu(rootNode, scriptEngine,
                                stringPipeEngine, parent);
        }
        if (rootNode.toElement().tagName() == "buttontable")  {
            rootGrid = pcGrid::buttonTable(rootNode, stringPipeEngine, parent);
            connect(rootGrid, SIGNAL(buttonClicked(QObject*)),
                    this,     SLOT(anyButtonClicked(QObject*)));
        }
        if (rootNode.toElement().tagName() == "hooks") {
            stringPipeEngine->setHooks(rootNode);
        }
        if (rootNode.toElement().tagName() == "statusbar") {
            statusbar = new pcStatusBar(&rootNode,stringPipeEngine, parent);
        }
        if (rootNode.toElement().tagName() == "engine") {
            scriptEngine = pcScriptEngine::create(&rootNode, parent);
        }
    }

    QDomNode empty;
    if (scriptEngine == NULL) {
        scriptEngine = new pcQScriptEngine(&empty, parent);
    }
    if (statusbar == NULL) {
        statusbar = new pcStatusBar(&empty,stringPipeEngine,parent);
    }

    QList<pcButton *> allNewPcButtons = parent->findChildren<pcButton *>();
    QList<pcButton *>::iterator b;
    for (b = allNewPcButtons.begin(); b < allNewPcButtons.end(); ++b) {
        keysToButtons[(*b)->text()] = (QObject *)(*b);
    }
    connect(lineEdit->lineEdit(), SIGNAL(returnPressed(void)),
            this, SLOT(lineEdit_returnPressed(void)));
    connect(lineEdit, SIGNAL(useKeyPress(QString, bool &)),
            this, SLOT(lineEdit_useKeyPress(QString, bool &)));

    updateLabels();
    statusbar->setStatusText("Engine loaded");
    delete doc;
}

QDomDocument* pcLayout::readDomFromFile(QString filename)
{
    QDomDocument *doc = new QDomDocument("buttontable");
    QFile file(filename);
    QString errorMsg;
    int errorLine;
    int errorColumn;

    if (!file.open(QIODevice::ReadOnly)) {
        QErrorMessage errorMessage;
        errorMessage.showMessage(QString("Can not open file ") + filename);
        errorMessage.exec();
    }
    if (!doc->setContent(&file, true, &errorMsg, &errorLine, &errorColumn)) {
        file.close();
        QErrorMessage errorMessage;
        errorMessage.showMessage(QString("Error while processing file ") + filename
                                 + QString(" at line ") + QString::number(errorLine)
                                 + QString(", column ") + QString::number(errorColumn)
                                 + QString(": ") + errorMsg);
        errorMessage.exec();
    }
    file.close();
    return(doc);
}

void pcLayout::updateLabels()
{
    QList<pcLabel *> allPcLabels = parent()->findChildren<pcLabel *>();
    QList<pcLabel *>::iterator l;
    for (l = allPcLabels.begin(); l < allPcLabels.end(); ++l) {
        (*l)->updateCaption();
    }

    QList<pcButton *> allPcButtons = parent()->findChildren<pcButton *>();
    QList<pcButton *>::iterator b;
    for (b = allPcButtons.begin(); b < allPcButtons.end(); ++b) {
        (*b)->updateCaption();
    }
}

void pcLayout::anyButtonClicked(QObject * obj)
{
    pcButton *action = (pcButton*)obj;
    macro->addMacroItem(action->objectName());
    action->updateLineEdit(lineEdit->lineEdit());
    QString display = lineEdit->lineEdit()->text();
    QString command;
    QString reply;
    lineEdit->storeStatesToScript();
    QString error("no error");
    if (action->getSend()) {        
        emit output("> "+lineEdit->currentText().trimmed());
        stringPipeEngine->eval("command = display;");
        stringPipeEngine->eval("globalpresend();");
        action->callHook("presend");
        command = stringPipeEngine->eval("command;");
        reply = scriptEngine->eval(command);
        stringPipeEngine->eval("reply = \""+reply+"\";");
        stringPipeEngine->eval("globalpostsend();");
        action->callHook("postsend");
        display = stringPipeEngine->eval("reply;");
    }
    stringPipeEngine->eval("display = \""+display+"\";");
    stringPipeEngine->eval("globalpreshow();");
    action->callHook("preshow");
    display = stringPipeEngine->eval("display;");
    if (action->getSend()) {
        emit output(display);
        lineEdit->addNewText(display);
    } else {
        lineEdit->lineEdit()->setText(display);
    }
    updateLabels();
    statusbar->setStatusText(error);
}

void pcLayout::lineEdit_returnPressed()
{
    anyButtonClicked(enter);
}

void pcLayout::lineEdit_useKeyPress(QString key, bool &inhibit)
{
    inhibit = false;
    if ((key != "") &&
        (keysToButtons.find(key) != keysToButtons.end()))
    {
        anyButtonClicked(keysToButtons[key]);
        inhibit = true;
    }
    updateLabels();
    statusbar->setStatusText("");
}

void pcLayout::showHelp()
{
    QMessageBox msgBox;
    msgBox.setTextFormat(Qt::RichText);
    msgBox.setText(layouthelp);
    msgBox.exec();
}

pcLayout::~pcLayout()
{
    QList<pcMenu *>::iterator m;
    for (m = menus.begin(); m < menus.end(); ++m) delete *m;
    delete lineEdit;
    delete rootGrid;
    delete statusbar;
    delete scriptEngine;
    delete stringPipeEngine;
    delete macro;
}
